import Interrupt from './interrupt'
import Base from '@uci/base'
import logger from '@uci-utils/logger'
let log = {}

// will more easily create a group of sbc pin interrupts

class Interrupts extends Base {

  constructor(pins, opts = {}) {
    super(opts)
    this.id = this.id || 'interrupts'
    this.pins = pins.map(pin => Number(pin))  // make sure actual numbers are passed
    this._interrupts = new Map()
    this.resetCmd = opts.resetCmd || 'interrupt.reset'
    log = logger({ name: 'interrupts', id: this.id, package:'@uci/interrupt', file:'src/interrupts.js'})
    this.amendConsumerCommands({reset:makefunc.bind(this,'reset')})
    this.amendSocketCommands({
      reset:makefunc.bind(this,'reset'),
      status:makefunc.bind(this,'status'),
      fire:makefunc.bind(this,'fire'),
      intervalReset:makefunc.bind(this,'intervalReset')
    })
    let pinopts = {}
    pins.forEach(pin => {
      // remove per pin opts and store
      pinopts[pin] = Object.assign({}, opts[pin])
      delete opts[pin]
    })
    pins.forEach(pin => {
      pinopts[pin] = Object.assign({}, opts, pinopts[pin])
      pinopts[pin].id = pinopts[pin].id || this.id + ':' + pin
      log.debug({ opts: pinopts[pin], method:'constructor', line:25, msg:`pin options for pin ${pin}`})
      this._interrupts.set(pin, new Interrupt(pin, pinopts[pin]))

      // TODO add socket and add methods to namespace if set

      // bubble up events from single interrupts
      const EVENTS=['log','connection:consumer', 'connection:socket']  // that should emit up from pin base
      EVENTS.forEach(event => {
        this.interrupt(pin).on(event, obj => {
          if (Object.prototype.toString.call(obj) !== '[object Object]') {
            let data=obj
            obj = {}
            obj.data = data
          }
          obj.pin = pin
          obj.id = pinopts[pin].id
          this.emit(event,obj)
        })
      })
    })

    this._interrupts.forEach( inter => {
      inter.on('interrupt', packet => {
        this.emit('interrupt',packet)
      })
      inter.on('reset', packet => {
        this.emit('reset',packet)
      })
      // overwrite single interrupt send
      inter.send = async (packet) => {
        return await this.send(packet) // send via common consumer
      }
    })

  } // end constructor

  interrupt(pin) { return this._interrupts.get(Number(pin)) } // get a handle to single interrupt

  async init() {
    super.init()
    return Promise.all(
      Array.from(this._interrupts).map(inter => {
        return inter[1].init()
      })
    )
  }

  // only adds consumer sockets to each interrupt to same socket/server
  // alternatively use listen handler and single socket
  async addInterSocket(name,type)  {
    if (type !=='s') {
      this._interrupts.forEach( inter => {
        inter.registerSocket(...arguments)
      })
    }
  }

  registerHook(func) {
    this._interrupts.forEach(inter => {
      inter.registerHook(func)
    })
  }

} // end Class

export default Interrupts

async function makefunc(fn, packet={}) {
  if (!packet.pin || packet.pin==='all') {
    let rpacket = {pins:[]}
    for (let inter of this._interrupts.entries()) {
      let ipacket = Object.assign({},packet)
      delete(ipacket.cmd);delete(ipacket._header)
      let res = await inter[1][fn](ipacket)
      res.pin = inter[0]
      rpacket.pins.push(res)
    }
    return rpacket
  }
  let pin = isNaN(Number(packet)) ? packet.pin : packet
  if (this._interrupts.has(Number(pin))) return await this.interrupt(packet.pin)[fn](packet)
}
